@@ -1,4 +1,4 @@
-spGLM <- function(formula, family="binomial", data = parent.frame(), coords, knots,
+spGLM <- function(formula, family="binomial", data = parent.frame(), coords, knots, amcmc,
                   starting, tuning, priors, cov.model, 
                   n.samples, sub.samples, verbose=TRUE, n.report=100, ...){
   
@@ -42,6 +42,45 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   ####################################################
   if(!family %in% c("binomial","poisson"))
     stop("error: family must be binomial or poisson")
+
+  ####################################################
+  ##sampling method
+  ####################################################
+  n.batch <- 0
+  batch.length <- 0
+  accept.rate <- 0
+    
+  if(missing(amcmc)){
+    
+    is.amcmc <- FALSE
+    if(missing(n.samples)){stop("error: n.samples need to be specified")}
+    
+  }else{
+    
+    is.amcmc <- TRUE
+
+    names(amcmc) <- tolower(names(amcmc))
+    
+    if(!"n.batch" %in% names(amcmc)){stop("error: n.batch must be specified in amcmc list")}
+    n.batch <- amcmc[["n.batch"]]
+    
+    if(!"batch.length" %in% names(amcmc)){stop("error: batch.length must be specified in amcmc list")}
+    batch.length <- amcmc[["batch.length"]]
+
+    if(!"accept.rate" %in% names(amcmc)){
+      warning("accept.rate was not specified in the amcmc list and was therefore set to the default 0.43")
+      accept.rate <- 0.43
+    }else{
+      accept.rate <- amcmc[["accept.rate"]]
+    }
+
+    n.samples <- n.batch*batch.length
+  }
+  
+  storage.mode(n.samples) <- "integer"
+  storage.mode(n.batch) <- "integer"
+  storage.mode(batch.length) <- "integer"
+  storage.mode(accept.rate) <- "double"
   
   ####################################################
   ##Distance matrices
@@ -63,7 +102,6 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   ####################
   ##Knots
   ####################
-
   is.pp <- FALSE
   modified.pp <- FALSE ##fixed for now
   
@@ -151,7 +189,6 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   ####################################################
   ##Starting values
   ####################################################
-
   beta.starting <- 0
   sigma.sq.starting <- 0
   phi.starting <- 0
@@ -264,60 +301,82 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   nu.tuning <- 0
   w.tuning <- 0
   
-  if(missing(tuning)){stop("error: tuning value vector for the spatial parameters must be specified")}
-  
-  names(tuning) <- tolower(names(tuning))
-
-  if(!"beta" %in% names(tuning)){stop("error: beta must be specified in tuning value list")}
-  beta.tuning <- tuning[["beta"]]
-
-  if(is.matrix(beta.tuning)){
-    if(nrow(beta.tuning) != p || ncol(beta.tuning) != p)
-      stop(paste("error: if beta tuning is a matrix, it must be of dimension ",p,sep=""))
+  if(!missing(tuning)){
     
-  }else if(is.vector(beta.tuning)){
-    if(length(beta.tuning) != p)
-      stop(paste("error: if beta tuning is a vector, it must be of length ",p,sep=""))
-
-    if(length(beta.tuning) > 1)
-      beta.tuning <- diag(beta.tuning)
+    names(tuning) <- tolower(names(tuning))
     
-  }else{
-    stop("error: beta tuning is misspecified")
-  }
-  
-  if(!"sigma.sq" %in% names(tuning)){stop("error: sigma.sq must be specified in tuning value list")}
-  sigma.sq.tuning <- tuning[["sigma.sq"]][1]
-  
-  if(!"phi" %in% names(tuning)){stop("error: phi must be specified in tuning value list")}
-  phi.tuning <- tuning[["phi"]][1]
+    if(!"beta" %in% names(tuning)){stop("error: beta must be specified in tuning value list")}
+    beta.tuning <- tuning[["beta"]]
     
-  if(cov.model == "matern"){
-    if(!"nu" %in% names(tuning)){stop("error: nu must be specified in tuning value list")}
-    nu.tuning <- tuning[["nu"]][1]
-  }    
-
-  if(!"w" %in% names(tuning)){stop("error: w must be specified in tuning value list")}
-  w.tuning <- tuning[["w"]]
-
-  if(is.pp){
-    if(length(w.tuning) == 1){
-      w.tuning <- tuning[["w"]][1]
-      w.tuning <- rep(w.tuning, m)
-    }else if(length(w.tuning) == m){
-      w.tuning <- tuning[["w"]]  
+    if(is.matrix(beta.tuning)){
+      if(nrow(beta.tuning) != p || ncol(beta.tuning) != p)
+        stop(paste("error: if beta tuning is a matrix, it must be of dimension ",p,sep=""))
+      
+      if(is.amcmc)
+        beta.tuning <- diag(beta.tuning)
+      
+    }else if(is.vector(beta.tuning)){
+      if(length(beta.tuning) != p)
+        stop(paste("error: if beta tuning is a vector, it must be of length ",p,sep=""))
+      
+      if(!is.amcmc)
+        beta.tuning <- diag(beta.tuning)
+      
     }else{
-      stop(paste("error: w in the tuning value list must be a scalar of length 1 or vector of length ",m," (i.e., the number of predictive process knots)",sep=""))
-    }    
-  }else{
-    if(length(w.tuning) == 1){
-      w.tuning <- tuning[["w"]][1]
-      w.tuning <- rep(w.tuning, n)
-    }else if(length(w.tuning) == n){
-      w.tuning <- tuning[["w"]]  
-    }else{
-      stop(paste("error: w in the tuning value list must be a scalar of length 1 or vector of length ",n,sep=""))
+      stop("error: beta tuning is misspecified")
     }
+    
+    if(!"sigma.sq" %in% names(tuning)){stop("error: sigma.sq must be specified in tuning value list")}
+    sigma.sq.tuning <- tuning[["sigma.sq"]][1]
+    
+    if(!"phi" %in% names(tuning)){stop("error: phi must be specified in tuning value list")}
+    phi.tuning <- tuning[["phi"]][1]
+    
+    if(cov.model == "matern"){
+      if(!"nu" %in% names(tuning)){stop("error: nu must be specified in tuning value list")}
+      nu.tuning <- tuning[["nu"]][1]
+    }    
+    
+    if(!"w" %in% names(tuning)){stop("error: w must be specified in tuning value list")}
+    w.tuning <- tuning[["w"]]
+    
+    if(is.pp){
+      if(length(w.tuning) == 1){
+        w.tuning <- tuning[["w"]][1]
+        w.tuning <- rep(w.tuning, m)
+      }else if(length(w.tuning) == m){
+        w.tuning <- tuning[["w"]]  
+      }else{
+        stop(paste("error: w in the tuning value list must be a scalar of length 1 or vector of length ",m," (i.e., the number of predictive process knots)",sep=""))
+      }    
+    }else{
+      if(length(w.tuning) == 1){
+        w.tuning <- tuning[["w"]][1]
+        w.tuning <- rep(w.tuning, n)
+      }else if(length(w.tuning) == n){
+        w.tuning <- tuning[["w"]]  
+      }else{
+        stop(paste("error: w in the tuning value list must be a scalar of length 1 or vector of length ",n,sep=""))
+      }
+    }
+    
+  }else{##no tuning provided
+    
+    if(!is.amcmc){
+      stop("error: tuning value list must be specified")
+    }
+    
+    beta.tuning <- rep(0.01,p)
+    phi.tuning <- 0.01
+    sigma.sq.tuning <- 0.01
+    nu.tuning <- 0.01
+    
+    if(is.pp){
+      w.tuning <- rep(0.01,m)
+    }else{
+      w.tuning <- rep(0.01,n)
+    }
+    
   }
   
   storage.mode(beta.tuning) <- "double"
@@ -328,13 +387,10 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   
   ####################################################
   ##Other stuff
-  ####################################################
-  if(missing(n.samples)){stop("error: n.samples need to be specified")}
-  
+  #################################################### 
   if(missing(sub.samples)){sub.samples <- c(1, n.samples, 1)}
   if(length(sub.samples) != 3 || any(sub.samples > n.samples) ){stop("error: sub.samples misspecified")}
   
-  storage.mode(n.samples) <- "integer"
   storage.mode(n.report) <- "integer"
   storage.mode(verbose) <- "integer"
 
@@ -344,22 +400,38 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   ####################################################
 
   if(is.pp){
-    
-    out <- .Call("spPPGLM", Y, X, p, n, coords.D, family,
-                 modified.pp, m, knots.D, coords.knots.D,               
-                 beta.prior, beta.Norm, sigma.sq.IG, nu.Unif, phi.Unif,
-                 phi.starting, sigma.sq.starting, nu.starting, beta.starting, w.starting,
-                 phi.tuning, sigma.sq.tuning, nu.tuning, beta.tuning, w.tuning,
-                 cov.model, n.samples, verbose, n.report)
+
+    if(!is.amcmc){
+      out <- .Call("spPPGLM", Y, X, p, n, coords.D, family,
+                   modified.pp, m, knots.D, coords.knots.D,               
+                   beta.prior, beta.Norm, sigma.sq.IG, nu.Unif, phi.Unif,
+                   phi.starting, sigma.sq.starting, nu.starting, beta.starting, w.starting,
+                   phi.tuning, sigma.sq.tuning, nu.tuning, beta.tuning, w.tuning,
+                   cov.model, n.samples, verbose, n.report)  
+    }else{
+      out <- .Call("spPPGLM_AMCMC", Y, X, p, n, coords.D, family,
+                   modified.pp, m, knots.D, coords.knots.D,               
+                   beta.prior, beta.Norm, sigma.sq.IG, nu.Unif, phi.Unif,
+                   phi.starting, sigma.sq.starting, nu.starting, beta.starting, w.starting,
+                   phi.tuning, sigma.sq.tuning, nu.tuning, beta.tuning, w.tuning,
+                   cov.model, n.batch, batch.length, accept.rate, verbose, n.report)
+    }
     
   }else{
     
-    out <- .Call("spGLM", Y, X, p, n, coords.D, family,
-                 beta.prior, beta.Norm, sigma.sq.IG, nu.Unif, phi.Unif,
-                 phi.starting, sigma.sq.starting, nu.starting, beta.starting, w.starting,
-                 phi.tuning, sigma.sq.tuning, nu.tuning, beta.tuning, w.tuning,
-                 cov.model, n.samples, verbose, n.report)
-    
+    if(!is.amcmc){
+      out <- .Call("spGLM", Y, X, p, n, coords.D, family,
+                   beta.prior, beta.Norm, sigma.sq.IG, nu.Unif, phi.Unif,
+                   phi.starting, sigma.sq.starting, nu.starting, beta.starting, w.starting,
+                   phi.tuning, sigma.sq.tuning, nu.tuning, beta.tuning, w.tuning,
+                   cov.model, n.samples, verbose, n.report)
+    }else{
+      out <- .Call("spGLM_AMCMC", Y, X, p, n, coords.D, family,
+                   beta.prior, beta.Norm, sigma.sq.IG, nu.Unif, phi.Unif,
+                   phi.starting, sigma.sq.starting, nu.starting, beta.starting, w.starting,
+                   phi.tuning, sigma.sq.tuning, nu.tuning, beta.tuning, w.tuning,
+                   cov.model, n.batch, batch.length, accept.rate, verbose, n.report)
+    }
   }
 
   out$coords <- coords
@@ -383,7 +455,12 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   out$sub.samples <- sub.samples
   out$recovered.effects <- TRUE
 
-  ##subsample 
+  if(is.amcmc){
+    out$tuning.w <- exp(out$tuning.w)
+    out$tuning <- exp(out$tuning)
+  }
+  
+  #subsample 
   out$sp.effects <- out$sp.effects[,seq(sub.samples[1], sub.samples[2], by=as.integer(sub.samples[3]))]
   if(is.pp){out$sp.effects.knots <- out$sp.effects.knots[,seq(sub.samples[1], sub.samples[2], by=as.integer(sub.samples[3]))]}
   out$p.samples <- mcmc(t(out$p.samples[,seq(sub.samples[1], sub.samples[2], by=as.integer(sub.samples[3]))]))
@@ -399,7 +476,7 @@ spGLM <- function(formula, family="binomial", data = parent.frame(), coords, kno
   }
     
   colnames(out$p.samples) <- col.names
-  
+
   class(out) <- "spGLM"
   out  
 }
